/**************************************************************************
 *
 *	project				 : qtplot
 *
 *	copyright            : (C) 2004 by Michael Neuroth
 *
 * ------------------------------------------------------------------------
 *
 *  $Source: $
 *
 *  $Revision: $
 *
 *	$Log: $
 *	
 *
 ***************************************************************************/
/***************************************************************************
 *																		   *
 * This file is part of the qtplot package (a gnuplot plotting frontend    *
 * for Qtopia and Qt)                                                      *
 *																		   *
 * Copyright (C) 2002 by Michael Neuroth.								   *
 *                                                                         *
 * This program is free software; you can redistribute it and/or modify    *
 * it under the terms of the GNU General Public License as published by    *
 * the Free Software Foundation; either version 2 of the License, or       *
 * (at your option) any later version.                                     *
 *                                                                         *
 ***************************************************************************/

#include "graphicoperation.h"

#include <qcanvas.h>

// ************************************************************************

GraphicEnv::GraphicEnv( QCanvas * pCanvas )
: m_pCanvas( pCanvas ),
  m_x( 0 ),
  m_y( 0 )
{
}

void GraphicEnv::ClearCanvas()
{
	m_pCanvas = 0;
}

bool GraphicEnv::IsCanvasOk() const
{
	return m_pCanvas != 0;
}

void GraphicEnv::SetBrush( const QBrush & aBrush )
{
	m_aActBrush = aBrush;
}

QBrush GraphicEnv::GetBrush() const
{
	return m_aActBrush;
}

void GraphicEnv::SetPen( const QPen & aPen )
{
	m_aActPen = aPen;
}

QPen GraphicEnv::GetPen() const
{
	return m_aActPen;
}

void GraphicEnv::SetPos( int x, int y )
{
	m_x = x;
	m_y = y;
}

void GraphicEnv::GetPos( int & x, int & y ) const
{
	x = m_x;
	y = m_y;
}

int	GraphicEnv::ScaleX( int x ) const
{
	return (QTOPIA_X_OFFSET+x)*m_pCanvas->width()/MAX_QTOPIA_WIDTH;
}

int	GraphicEnv::ScaleY( int y ) const
{
	return (MAX_QTOPIA_HEIGHT-y-QTOPIA_Y_OFFSET)*m_pCanvas->height()/MAX_QTOPIA_HEIGHT;
}

int GraphicEnv::RelScaleX( int x ) const
{
	return x*m_pCanvas->width()/MAX_QTOPIA_WIDTH;
}

int GraphicEnv::RelScaleY( int y ) const
{
	return y*m_pCanvas->height()/MAX_QTOPIA_HEIGHT;
}

QCanvas * GraphicEnv::GetCanvas() const
{
	return m_pCanvas;
}

// ************************************************************************

GraphicOperation::~GraphicOperation()
{
}

void GraphicOperation::Draw( GraphicEnv & env )
{
	// dummy implementation
}

void GraphicOperation::Clear( GraphicEnv & env )
{
	// dummy implementation
}

// ************************************************************************

GrOpMove::GrOpMove( int x, int y )
{
	m_x = x;
	m_y = y;
}

int GrOpMove::GetX( GraphicEnv & env ) const
{
	return env.ScaleX( m_x );
}

int GrOpMove::GetY( GraphicEnv & env ) const
{
	return env.ScaleY( m_y );
}

void GrOpMove::Draw( GraphicEnv & env )
{
	env.SetPos( m_x, m_y );
}

// ************************************************************************

GrOpPoint::GrOpPoint( int x, int y, int iPoint )
: GrOpMove( x, y ),
  m_pPoint( 0 )
{
	m_iPoint = iPoint;
}

GrOpPoint::~GrOpPoint()
{
	delete m_pPoint;
}

void GrOpPoint::Draw( GraphicEnv & env )
{
	if( !m_pPoint )
	{
		m_pPoint = new QCanvasEllipse( env.GetCanvas() );
	}

	m_pPoint->move( GetX( env ), GetY( env ) );
	m_pPoint->setSize( env.RelScaleX( POINT_X ), env.RelScaleY( POINT_Y ) );
	m_pPoint->setPen( env.GetPen() );
	m_pPoint->setBrush( env.GetPen().color() );
	m_pPoint->show();

	// save the actual position for the next plot step...
	GrOpMove::Draw( env );
}

void GrOpPoint::Clear( GraphicEnv & env )
{
	delete m_pPoint;
	m_pPoint = 0;
}

// ************************************************************************

GrOpVector::GrOpVector( int x, int y )
: GrOpMove( x, y ),
  m_pLine( 0 )
{
}

GrOpVector::~GrOpVector()
{
	delete m_pLine;
}

void GrOpVector::Draw( GraphicEnv & env )
{
	if( !m_pLine )
	{
		m_pLine = new QCanvasLine( env.GetCanvas() );
	}

	int xOld, yOld;
	env.GetPos( xOld, yOld );

	m_pLine->setPoints( env.ScaleX( xOld ), env.ScaleY( yOld ), GetX( env ), GetY( env ) );
	m_pLine->setPen( env.GetPen() );
	m_pLine->show();

	// save the actual position for the next plot step...
	GrOpMove::Draw( env );
}

void GrOpVector::Clear( GraphicEnv & env )
{
	delete m_pLine;
	m_pLine = 0;
}

// ************************************************************************

GrOpLinetype::GrOpLinetype( int iLinetype )
{
	m_iLinetype = iLinetype;
}

void GrOpLinetype::Draw( GraphicEnv & env )
{
	QBrush aActBrush = env.GetBrush();
	QPen aActPen = env.GetPen();

//	aActBrush.setColor( QColor( 0, 0, 0 ) );
//	aActBrush.setStyle( QBrush::NoBrush );
//	env.SetBrush( aActBrush );

	switch( m_iLinetype )
	{
		case -3:
			break;
		case -2:
			// border of the plot
			aActPen.setColor( QColor( 0, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case -1:
			// for the x and y axis
			aActPen.setColor( QColor( 0, 0, 255 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		case 0:
		case 1:
		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
			aActPen.setColor( QColor( 255, 0, 0 ) );
			aActPen.setWidth( 1 );
			aActPen.setStyle( QPen::SolidLine );
			env.SetPen( aActPen );
			break;
		default: 
			;
	}
}

// ************************************************************************

GrOpPutText::GrOpPutText( int x, int y, const char * sText )
: GrOpMove( x, y ),
  m_pText( 0 )
{
	m_sText = sText;
}

GrOpPutText::~GrOpPutText()
{
	delete m_pText;
}

void GrOpPutText::Draw( GraphicEnv & env )
{
	if( !m_pText )
	{
		m_pText = new QCanvasText( m_sText, env.GetCanvas() );
	}

	m_pText->move( GetX( env ), GetY( env ) );
	m_pText->setColor( env.GetPen().color() );
	m_pText->show();
}

void GrOpPutText::Clear( GraphicEnv & env )
{
	delete m_pText;
	m_pText = 0;
}
